/*
 * ProActive Parallel Suite(TM):
 * The Open Source library for parallel and distributed
 * Workflows & Scheduling, Orchestration, Cloud Automation
 * and Big Data Analysis on Enterprise Grids & Clouds.
 *
 * Copyright (c) 2007 - 2017 ActiveEon
 * Contact: contact@activeeon.com
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation: version 3 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 */
package org.ow2.proactive.resourcemanager.common.event;

import org.objectweb.proactive.annotation.PublicAPI;


/**
 * Enumeration of all events generated by the Resource Manager,
 * and received by Monitors, which permits to probe RM activity.
 *
 * @see org.ow2.proactive.resourcemanager.frontend.RMMonitoring
 *
 * @author The ProActive Team
 * @since ProActive Scheduling 0.9
 *
 */
@PublicAPI
public enum RMEventType {

    /**
     * Resource manager has been shutdown.
     */
    SHUTDOWN("shutdown"),
    /**
     * Resource manager cessation as been asked, so Resource manager performs 
     * shutting down operations.  
     */
    SHUTTING_DOWN("shutting down"),
    /**
     * Resource manager has been launched. 
     */
    STARTED("started"),
    /**
     * The node source has been defined in the resource manager.
     */
    NODESOURCE_DEFINED("defined"),
    /**
     * The node source has been deployed, i.e. its nodes are being acquired.
     */
    NODESOURCE_CREATED("created"),
    /**
     * The node source has been updated, i.e. its parameters have changed
     * while the node source is deployed.
     */
    NODESOURCE_UPDATED("updated"),
    /**
     * The node source has been shutdown, i.e. its nodes have been removed
     * and it does not try to acquire nodes anymore.
     */
    NODESOURCE_SHUTDOWN("shutdown"),
    /**
     * A node source has been removed from resource manager.
     */
    NODESOURCE_REMOVED("removed"),
    /**
    /**
     * A new node has been added to Resource manager, and is ready to
     * be provided to a RM user.
     */
    NODE_ADDED("added"),
    /**
     * Node state has been changed.
     */
    NODE_STATE_CHANGED("changed"),
    /**
     * A node has been removed from Resource manager
     */
    NODE_REMOVED("removed");

    private final String description;

    RMEventType(String description) {
        this.description = description;
    }

    public String getDescription() {
        return this.description;
    }

}
