/*
 * ProActive Parallel Suite(TM):
 * The Open Source library for parallel and distributed
 * Workflows & Scheduling, Orchestration, Cloud Automation
 * and Big Data Analysis on Enterprise Grids & Clouds.
 *
 * Copyright (c) 2007 - 2017 ActiveEon
 * Contact: contact@activeeon.com
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation: version 3 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 */
package org.ow2.proactive.resourcemanager.nodesource.common;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;


/**
 *
 * Annotation for automatic GUI construction.
 * This is a marker for GUI builder to include an annotated field into the GUI.
 * Initial value of the field will be used in GUI as a string.
 *
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.FIELD })
public @interface Configurable {

    /**
     * Field's description
     */
    String description() default "";

    /**
     * Password field. Should not be displayed as a plain text
     */
    boolean password() default false;

    /**
     * Credential field. Based on this parameter special gui is generated
     * to simplify the credential creation.
     */
    boolean credential() default false;

    /**
     * Field represents a file. File browser will be added to the GUI.
     * A file will be loaded and converted to a byte array.
     */
    boolean fileBrowser() default false;

    /**
     * Field is a multiline text area.
     */
    boolean textArea() default false;

    /**
     * Field is a checkbox
     */
    boolean checkbox() default false;

    /**
     * Dynamic fields can be edited while the node source is deployed.
     */
    boolean dynamic() default false;

    /**
     * Section selector which is used to group parameters
     * and show description on top of the group
     */
    int sectionSelector() default 0;

    boolean important() default false;
}
