/*
 * ProActive Parallel Suite(TM):
 * The Open Source library for parallel and distributed
 * Workflows & Scheduling, Orchestration, Cloud Automation
 * and Big Data Analysis on Enterprise Grids & Clouds.
 *
 * Copyright (c) 2007 - 2017 ActiveEon
 * Contact: contact@activeeon.com
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation: version 3 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 */
package org.ow2.proactive.scheduler.common.exception;

import org.objectweb.proactive.annotation.PublicAPI;


/**
 * Exception generated by the scheduler JobId nextId() method informing that max Id has been reached.<br>
 * In this case, the Scheduler stops, and administrator may check what to do with the Database.<br>
 * For information, max Id is the Integer.MAX_VALUE.
 *
 * @author The ProActive Team
 * @since ProActive Scheduling 1.0
 */
@PublicAPI
public class MaxJobIdReachedException extends InternalException {
    /**
     * Create a new instance of MaxJobIdReachedException with the given message.
     *
     * @param msg the message to attach.
     */
    public MaxJobIdReachedException(String msg) {
        super(msg);
    }

    /**
     * Create a new instance of MaxJobIdReachedException.
     */
    public MaxJobIdReachedException() {
        super();
    }

    /**
     * Create a new instance of MaxJobIdReachedException with the given message and cause
     *
     * @param msg the message to attach.
     * @param cause the cause of the exception.
     */
    public MaxJobIdReachedException(String msg, Throwable cause) {
        super(msg, cause);
    }

    /**
     * Create a new instance of MaxJobIdReachedException with the given cause.
     *
     * @param cause the cause of the exception.
     */
    public MaxJobIdReachedException(Throwable cause) {
        super(cause);
    }
}
