/*
 * ProActive Parallel Suite(TM):
 * The Open Source library for parallel and distributed
 * Workflows & Scheduling, Orchestration, Cloud Automation
 * and Big Data Analysis on Enterprise Grids & Clouds.
 *
 * Copyright (c) 2007 - 2017 ActiveEon
 * Contact: contact@activeeon.com
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation: version 3 of
 * the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * If needed, contact us to obtain a release under GPL Version 2 or 3
 * or a different license than the AGPL.
 */
package functionaltests.workflow;

import static org.junit.Assert.assertFalse;

import java.io.Serializable;

import org.junit.Assert;
import org.junit.Test;
import org.ow2.proactive.scheduler.common.job.JobId;
import org.ow2.proactive.scheduler.common.job.JobResult;
import org.ow2.proactive.scheduler.common.job.TaskFlowJob;
import org.ow2.proactive.scheduler.common.task.JavaTask;
import org.ow2.proactive.scheduler.common.task.TaskResult;
import org.ow2.proactive.scheduler.common.task.executable.JavaExecutable;

import functionaltests.utils.SchedulerFunctionalTestNoRestart;
import functionaltests.utils.SchedulerTHelper;


/**
 * @author ActiveEon Team
 * @since 22/10/2018
 */
public class TestResultMap extends SchedulerFunctionalTestNoRestart {

    private final String JOB_NAME = this.getClass().getSimpleName();

    public static class TestResultMapTask extends JavaExecutable {

        public static final String MYVAR = "myResultMapVar";

        public static final String MYVALUE = "myResultMapValue";

        @Override
        public Serializable execute(TaskResult... results) throws Throwable {
            getResultMap().put(MYVAR, MYVALUE);
            return true;
        }

    }

    @Test
    public void testResultMap() throws Throwable {
        testRegularJob();
    }

    private TaskFlowJob createJob() throws Exception {
        TaskFlowJob job = new TaskFlowJob();
        job.setName(JOB_NAME);

        JavaTask javaTask = new JavaTask();
        javaTask.setExecutableClassName(TestResultMapTask.class.getName());
        String TASK_NAME = "test_result_map";
        javaTask.setName(TASK_NAME);

        job.addTask(javaTask);

        return job;
    }

    public void testRegularJob() throws Throwable {

        JobId jobId = schedulerHelper.submitJob(createJob());
        SchedulerTHelper.log("Job submitted, id " + jobId.toString());
        schedulerHelper.waitForEventJobFinished(jobId);
        JobResult jobResult = schedulerHelper.getJobResult(jobId);
        assertFalse(schedulerHelper.getJobResult(jobId).getResultMap().isEmpty());
        Assert.assertEquals(jobResult.getResultMap().get("myResultMapVar"), "myResultMapValue");
    }

}
